//
//    Copyright 2005, Thomas C. McDermott, N5EG
//    This file is part of SIGGEN - the Signal Generator program.
//
//    SIGGEN is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    SIGGEN is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SIGGEN, if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
#pragma once

#include "stdafx.h"
#include "FrequencySelector.h"
#include <math.h>

#using <mscorlib.dll>
#using <System.dll>

using namespace System;
using namespace System::Windows::Forms;
using namespace System::IO;


// Convert from frequency to DDS accumulator value and vice versa.
// Fclk = RefMult * RefFreq
// Fout = Fclk * DDSAccum / 2^48
// DDSAccum = Fout * 2^48 / Fclk

long long unsigned FrequencySelector::FreqToDDSCorr(double frequency1)	// Convert frequency to 48-bit DDS accumulator
{

	// New 11-20-2007
	// Adjust for error of the internal reference frequency

	double frequency = frequency1;

	if(InternalRef)				// if we are using the internal reference oscillator
		frequency -= (double)internalRefFreqError * (frequency / 100000000.0);


	frequency *= 4294967296.0;	// Multiply by 2^32
	frequency /= FClk;			// divide by RefClk
	frequency *= 65536.0;	    // Multiply by 2^16
	
	long long unsigned divisor = Convert::ToInt64(frequency);

	return (divisor);

};

long long unsigned FrequencySelector::FreqToDDS(double frequency)	// Convert frequency to 48-bit DDS divisor without correction
{
	frequency *= 4294967296.0;	// Multiply by 2^32
	frequency /= FClk;			// divide by RefClk
	frequency *= 65536.0;	    // Multiply by 2^16
	
	long long unsigned divisor = Convert::ToInt64(frequency);

	return (divisor);

};


double FrequencySelector::DDSToFreq(long long unsigned accum)	// Convert DDS accumulator to frequency
{

	double Freq = Convert::ToDouble(accum);

	Freq /= 16777216.0;		// divide by 2^24
	Freq *= FClk;			// mult by RefClk
	Freq /= 16777216.0;		// divide again by 2^24
	
	return (Freq);
};



void FrequencySelector::Save(String ^ StartUpPath)					// Save values to file
{
			// Write out the settings to a file so we can pre-load them next time
			FileStream^ fs;
			BinaryWriter^ bw;

			String^ filename = String::Concat(StartUpPath, "\\sgen.dat");
			try
			{
				// Create a filestream and binary writer

				fs = gcnew FileStream(filename, FileMode::Create, FileAccess::Write);
				bw = gcnew BinaryWriter(fs);
			}
			catch(System::Exception^ pe)
			{
				MessageBox::Show("Error. Could not Write Signal Generator Data file.", pe->Message,
					MessageBoxButtons::OK, MessageBoxIcon::Error);
				return;
			}

			// Define header to match file identifying type and version

			String^ recognized = gcnew String("SGEN Reference Settings Data Set Version 1.0.1");
			bw->Write(recognized);		// put string header on outfile

			// write the Reference selection constants
			bw->Write(refFreq);
			bw->Write(refMult);
			bw->Write(internalRef);
			bw->Write(internalRefFreqError);

			fs->Flush();
			fs->Close();

			if(bw)
				bw->Close();

			return;

};
void FrequencySelector::Load(String^ StartUpPath)					// Load values from file
{
		// Try to read in the sgen.dat file to pre-load the Reference  constants
		// If the file does not exist, use default values (Internal, 24 MHz, 12x)

		FileStream^ fs;
		BinaryReader^ br;

		internalRef = true;		// use default reference parameters until params sucessfully retrieved

		String^ filename = String::Concat(StartUpPath, "\\sgen.dat");
		try
		{
			// Create a filestream and binary reader

			fs = gcnew FileStream(filename, FileMode::Open, FileAccess::Read);
			br = gcnew BinaryReader(fs);
		}
		catch(System::Exception^ /* pe */)
		{
			// File not found - use internal Reference Selection
			if(fs)
				fs->Close();
			if(br)
				br->Close();
			return;
		}

		// Define header to match file identifying type and version

		String^ recognized = gcnew String("SGEN Reference Settings Data Set Version 1.0.1");
		String^ header;

		header = br->ReadString();		// get string header on infile
		if (String::Compare(recognized, header) != 0)
		{
			MessageBox::Show(
				"sgen.dat file is incompatible type or version.\n\rExpecting sgen Version 1.0.1",
				"Error", MessageBoxButtons::OK, MessageBoxIcon::Error);

			if(fs)
				fs->Close();
			if(br)
				br->Close();

			return;
		}
	
		// read in the Reference constants
		try
		{
			refFreq = br->ReadDouble();
			refMult = br->ReadInt32();
			internalRef = br->ReadBoolean();
			internalRefFreqError = br->ReadInt32();
		}
		catch(System::Exception^ pe)
		{
			MessageBox::Show("Error trying to load Signal Generator reference constants File.\n\r"
				"Delete the file:   sgen.dat   and re-set the reference selections", pe->Message,
				MessageBoxButtons::OK, MessageBoxIcon::Error);

			if(fs)
				fs->Close();
			if(br)
				br->Close();

			return;
		}

		if(fs)
			fs->Close();
		if(br)
			br->Close();

		return;
};

// BUGBUG : need to clean up the getter and setter properties below
// Object properties
//int FrequencySelector::get_refMult(){ return(RefMult); };		

//double FrequencySelector::get_refFreq(){ return(RefFreq); };

//bool FrequencySelector::get_internalRef(){ return(InternalRef); };

//double FrequencySelector::get_fClk(){ return(FClk); };
//void FrequencySelector::set_refMult(int m)
//{
//		RefMult = m;
//		FClk = RefFreq * (double)RefMult;
//};

//void FrequencySelector::set_refFreq(double f)
//{
//		RefFreq = f;
//		FClk = RefFreq * (double)RefMult;
//};

//void FrequencySelector::set_internalRef(bool flag)
//{
//		InternalRef = flag;
//		if (InternalRef)
//		{
//			RefMult = 12;
//			RefFreq = 24000000.0;
//			FClk = RefFreq * (double)RefMult;
//		}
//};
//void FrequencySelector::set_internalRefFreqError(int error)
//{
//		InternalRefFreqError = error;
//};
//int FrequencySelector::get_internalRefFreqError(){ return(InternalRefFreqError); };


//#define MAX_DDS_LEVEL 4095	// maximum level possible (about = 1.5 dbm)
#define MAX_DDS_LEVEL 3446		// 3446 is about 0 dbm.
#define CENTERPHASE	1819		// guess for mid-readng for phase detectors
#define RAD2DEGR 180.0/3.14159265358	// Converts radians to degrees
#define DEGR2RAD 3.14159265358/180.0		// Converts degrees to radians


// Convert Log Tx Level to linear value needed by DAC register
unsigned short TxLevLinear(float tranLevel)
{
	// 0 dbm is the max level. Corresponds to 4095 decimal for IDAC control register.
	// 0 dbm -> 4095     -20 dbm -> 410    -40 dbm -> 41

	unsigned short linlevel = (unsigned short)((double)MAX_DDS_LEVEL * Math::Pow(10.0,((tranLevel)/20.0)) + 0.5);
	return (linlevel);
};
// Convert linear DAC level to dBm.  4095 = 0 dBm.
float dBFromLinear(int DAClevel)
{
	float dB = (float)(20.0 * Math::Log10((float(DAClevel)/(float)MAX_DDS_LEVEL)));
	return (dB);
};

float IQtoDegrees(int I, int Q)		// convert I/Q ADC reading to degrees
{
	float degrees;

	degrees = (float)(RAD2DEGR * atan2((double)(CENTERPHASE - Q),(double)(I - CENTERPHASE)));
	if (degrees > 180.0)
		degrees -= 360.0;

	return(degrees);
};

